@@ -138,6 +138,8 @@ G_DEFINE_ABSTRACT_TYPE (GstObject, gst_object, G_TYPE_INITIALLY_UNOWNED);
 static void
 gst_object_constructed (GObject * object)
 {
+  GST_OBJECT_FLAG_SET (object, GST_OBJECT_FLAG_CONSTRUCTED);
+
   GST_TRACER_OBJECT_CREATED (GST_OBJECT_CAST (object));
 
   ((GObjectClass *) gst_object_parent_class)->constructed (object);
@@ -153,7 +155,8 @@ gst_object_class_init (GstObjectClass * klass)
 
   properties[PROP_NAME] =
       g_param_spec_string ("name", "Name", "The name of the object", NULL,
-      G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS);
+      G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS |
+      GST_PARAM_DOC_SHOW_DEFAULT);
 
   /**
    * GstObject:parent:
@@ -166,7 +169,8 @@ gst_object_class_init (GstObjectClass * klass)
    */
   properties[PROP_PARENT] =
       g_param_spec_object ("parent", "Parent", "The parent of the object",
-      GST_TYPE_OBJECT, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      GST_TYPE_OBJECT,
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | GST_PARAM_DOC_SHOW_DEFAULT);
 
   g_object_class_install_properties (gobject_class, PROP_LAST, properties);
 
@@ -184,8 +188,7 @@ gst_object_class_init (GstObjectClass * klass)
       g_signal_new ("deep-notify", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE | G_SIGNAL_DETAILED |
       G_SIGNAL_NO_HOOKS, G_STRUCT_OFFSET (GstObjectClass, deep_notify), NULL,
-      NULL, g_cclosure_marshal_generic, G_TYPE_NONE, 2, GST_TYPE_OBJECT,
-      G_TYPE_PARAM);
+      NULL, NULL, G_TYPE_NONE, 2, GST_TYPE_OBJECT, G_TYPE_PARAM);
 
   klass->path_string_separator = "/";
 
@@ -295,11 +298,35 @@ gst_object_ref_sink (gpointer object)
   return g_object_ref_sink (object);
 }
 
+/**
+ * gst_clear_object: (skip)
+ * @object_ptr: a pointer to a #GstObject reference
+ *
+ * Clears a reference to a #GstObject.
+ *
+ * @object_ptr must not be %NULL.
+ *
+ * If the reference is %NULL then this function does nothing.
+ * Otherwise, the reference count of the object is decreased using
+ * gst_object_unref() and the pointer is set to %NULL.
+ *
+ * A macro is also included that allows this function to be used without
+ * pointer casts.
+ *
+ * Since: 1.16
+ **/
+#undef gst_clear_object
+void
+gst_clear_object (GstObject ** object_ptr)
+{
+  g_clear_pointer (object_ptr, gst_object_unref);
+}
+
 /**
  * gst_object_replace:
  * @oldobj: (inout) (transfer full) (nullable): pointer to a place of
  *     a #GstObject to replace
- * @newobj: (transfer none) (allow-none): a new #GstObject
+ * @newobj: (transfer none) (nullable): a new #GstObject
  *
  * Atomically modifies a pointer to point to a new object.
  * The reference count of @oldobj is decreased and the reference count of
@@ -324,7 +351,7 @@ gst_object_replace (GstObject ** oldobj, GstObject * newobj)
       newobj ? G_OBJECT (newobj)->ref_count : 0);
 #endif
 
-  oldptr = g_atomic_pointer_get ((gpointer *) oldobj);
+  oldptr = (GstObject *) g_atomic_pointer_get ((gpointer *) oldobj);
 
   if (G_UNLIKELY (oldptr == newobj))
     return FALSE;
@@ -333,7 +360,7 @@ gst_object_replace (GstObject ** oldobj, GstObject * newobj)
     gst_object_ref (newobj);
 
   while (G_UNLIKELY (!g_atomic_pointer_compare_and_exchange ((gpointer *)
-              oldobj, oldptr, newobj))) {
+              oldobj, (gpointer) oldptr, newobj))) {
     oldptr = g_atomic_pointer_get ((gpointer *) oldobj);
     if (G_UNLIKELY (oldptr == newobj))
       break;
@@ -570,10 +597,42 @@ had_parent:
   }
 }
 
+static gboolean
+gst_object_set_name_intern (GstObject * object, const gchar * name)
+{
+  gboolean result;
+
+  GST_OBJECT_LOCK (object);
+
+  /* parented objects cannot be renamed */
+  if (G_UNLIKELY (object->parent != NULL))
+    goto had_parent;
+
+  if (name != NULL) {
+    g_free (object->name);
+    object->name = g_strdup (name);
+    GST_OBJECT_UNLOCK (object);
+    result = TRUE;
+  } else {
+    GST_OBJECT_UNLOCK (object);
+    result = gst_object_set_name_default (object);
+  }
+
+  return result;
+
+  /* error */
+had_parent:
+  {
+    GST_WARNING ("parented objects can't be renamed");
+    GST_OBJECT_UNLOCK (object);
+    return FALSE;
+  }
+}
+
 /**
  * gst_object_set_name:
  * @object: a #GstObject
- * @name: (allow-none): new name of object
+ * @name: (nullable): new name of object
  *
  * Sets the name of @object, or gives @object a guaranteed unique
  * name (if @name is %NULL).
@@ -593,32 +652,9 @@ gst_object_set_name (GstObject * object, const gchar * name)
 
   g_return_val_if_fail (GST_IS_OBJECT (object), FALSE);
 
-  GST_OBJECT_LOCK (object);
-
-  /* parented objects cannot be renamed */
-  if (G_UNLIKELY (object->parent != NULL))
-    goto had_parent;
-
-  if (name != NULL) {
-    g_free (object->name);
-    object->name = g_strdup (name);
-    GST_OBJECT_UNLOCK (object);
-    result = TRUE;
-  } else {
-    GST_OBJECT_UNLOCK (object);
-    result = gst_object_set_name_default (object);
-  }
-
-  g_object_notify (G_OBJECT (object), "name");
+  if ((result = gst_object_set_name_intern (object, name)))
+    g_object_notify_by_pspec (G_OBJECT (object), properties[PROP_NAME]);
   return result;
-
-  /* error */
-had_parent:
-  {
-    GST_WARNING ("parented objects can't be renamed");
-    GST_OBJECT_UNLOCK (object);
-    return FALSE;
-  }
 }
 
 /**
@@ -901,7 +937,7 @@ gst_object_set_property (GObject * object, guint prop_id,
 
   switch (prop_id) {
     case PROP_NAME:
-      gst_object_set_name (gstobject, g_value_get_string (value));
+      gst_object_set_name_intern (gstobject, g_value_get_string (value));
       break;
     case PROP_PARENT:
       gst_object_set_parent (gstobject, g_value_get_object (value));
@@ -1308,7 +1344,7 @@ gst_object_remove_control_binding (GstObject * object,
  *
  * Gets the value for the given controlled property at the requested time.
  *
- * Returns: (nullable): the GValue of the property at the given time,
+ * Returns: (transfer full) (nullable): the GValue of the property at the given time,
  * or %NULL if the property isn't controlled.
  */
 GValue *
@@ -1338,7 +1374,7 @@ gst_object_get_value (GstObject * object, const gchar * property_name,
  * @timestamp: the time that should be processed
  * @interval: the time spacing between subsequent values
  * @n_values: the number of values
- * @values: array to put control-values in
+ * @values: (array length=n_values): array to put control-values in
  *
  * Gets a number of values for the given controlled property starting at the
  * requested time. The array @values need to hold enough space for @n_values of
@@ -1424,7 +1460,7 @@ gst_object_get_g_value_array (GstObject * object, const gchar * property_name,
  *
  * Obtain the control-rate for this @object. Audio processing #GstElement
  * objects will use this rate to sub-divide their processing loop and call
- * gst_object_sync_values() inbetween. The length of the processing segment
+ * gst_object_sync_values() in between. The length of the processing segment
  * should be up to @control-rate nanoseconds.
  *
  * If the @object is not under property control, this will return
@@ -1450,7 +1486,7 @@ gst_object_get_control_rate (GstObject * object)
  *
  * Change the control-rate for this @object. Audio processing #GstElement
  * objects will use this rate to sub-divide their processing loop and call
- * gst_object_sync_values() inbetween. The length of the processing segment
+ * gst_object_sync_values() in between. The length of the processing segment
  * should be up to @control-rate nanoseconds.
  *
  * The control-rate should not change if the element is in %GST_STATE_PAUSED or
