@@ -24,11 +24,28 @@ typedef struct {
   reference_t last;
 } chain_t;
 
+typedef struct {
+  gchar *factory_name;
+  GSList *values;
+  GSList *presets;
+} element_t;
+
+/* Filled from a bus sync handler if a error message is posted during the
+ * construction of the chain.
+ * The mutex is necessary as the bus could -- at least in principle -- have
+ * messages posted from several threads simultaneously. */
+typedef struct {
+  GMutex mutex;
+  gchar *reason; /* owned by the struct */
+} reason_receiver_t;
+
+
 typedef struct _graph_t graph_t;
 struct _graph_t {
   chain_t *chain; /* links are supposed to be done now */
   GSList *links;
   GError **error;
+  reason_receiver_t *error_probable_reason_receiver;
   GstParseContext *ctx; /* may be NULL */
   GstParseFlags flags;
 };
@@ -55,19 +72,25 @@ G_GNUC_INTERNAL  link_t *__gst_parse_link_new (void);
 G_GNUC_INTERNAL  void	__gst_parse_link_free (link_t *data);
 G_GNUC_INTERNAL  chain_t *__gst_parse_chain_new (void);
 G_GNUC_INTERNAL  void	__gst_parse_chain_free (chain_t *data);
+G_GNUC_INTERNAL  element_t *__gst_parse_element_new (void);
+G_GNUC_INTERNAL  void	__gst_parse_element_free (element_t *data);
 #  define gst_parse_strdup __gst_parse_strdup
 #  define gst_parse_strfree __gst_parse_strfree
 #  define gst_parse_link_new __gst_parse_link_new
 #  define gst_parse_link_free __gst_parse_link_free
 #  define gst_parse_chain_new __gst_parse_chain_new
 #  define gst_parse_chain_free __gst_parse_chain_free
+#  define gst_parse_element_new __gst_parse_element_new
+#  define gst_parse_element_free __gst_parse_element_free
 #else /* __GST_PARSE_TRACE */
 #  define gst_parse_strdup g_strdup
 #  define gst_parse_strfree g_free
-#  define gst_parse_link_new() g_slice_new0 (link_t)
-#  define gst_parse_link_free(l) g_slice_free (link_t, l)
-#  define gst_parse_chain_new() g_slice_new0 (chain_t)
-#  define gst_parse_chain_free(c) g_slice_free (chain_t, c)
+#  define gst_parse_link_new() g_new0 (link_t, 1)
+#  define gst_parse_link_free(l) g_free (l)
+#  define gst_parse_chain_new() g_new0 (chain_t, 1)
+#  define gst_parse_chain_free(c) g_free (c)
+#  define gst_parse_element_new() g_new0 (element_t, 1)
+#  define gst_parse_element_free(e) g_free (e)
 #endif /* __GST_PARSE_TRACE */
 
 static inline void
