@@ -27,6 +27,8 @@
 #define GST_BIT_WRITER_DISABLE_INLINES
 #include "gstbitwriter.h"
 
+#include "gst/glib-compat-private.h"
+
 /**
  * SECTION:gstbitwriter
  * @title: GstBitWriter
@@ -49,7 +51,7 @@
 GstBitWriter *
 gst_bit_writer_new (void)
 {
-  GstBitWriter *ret = g_slice_new0 (GstBitWriter);
+  GstBitWriter *ret = g_new0 (GstBitWriter, 1);
 
   ret->owned = TRUE;
   ret->auto_grow = TRUE;
@@ -70,7 +72,7 @@ gst_bit_writer_new (void)
 GstBitWriter *
 gst_bit_writer_new_with_size (guint size, gboolean fixed)
 {
-  GstBitWriter *ret = g_slice_new0 (GstBitWriter);
+  GstBitWriter *ret = g_new0 (GstBitWriter, 1);
 
   gst_bit_writer_init_with_size (ret, size, fixed);
   return ret;
@@ -78,7 +80,7 @@ gst_bit_writer_new_with_size (guint size, gboolean fixed)
 
 /**
  * gst_bit_writer_new_with_data: (skip)
- * @data: Memory area for writing
+ * @data: (array length=size) (transfer none): Memory area for writing
  * @size: Size of @data in bytes
  * @initialized: if %TRUE the complete data can be read from the beginning
  *
@@ -93,7 +95,7 @@ gst_bit_writer_new_with_size (guint size, gboolean fixed)
 GstBitWriter *
 gst_bit_writer_new_with_data (guint8 * data, guint size, gboolean initialized)
 {
-  GstBitWriter *ret = g_slice_new0 (GstBitWriter);
+  GstBitWriter *ret = g_new0 (GstBitWriter, 1);
 
   gst_bit_writer_init_with_data (ret, data, size, initialized);
 
@@ -200,7 +202,7 @@ gst_bit_writer_reset_and_get_data (GstBitWriter * bitwriter)
 
   data = bitwriter->data;
   if (bitwriter->owned)
-    data = g_memdup (data, bitwriter->bit_size >> 3);
+    data = g_memdup2 (data, GST_ROUND_UP_8 (bitwriter->bit_size) >> 3);
   gst_bit_writer_reset (bitwriter);
 
   return data;
@@ -223,16 +225,19 @@ gst_bit_writer_reset_and_get_buffer (GstBitWriter * bitwriter)
   GstBuffer *buffer;
   gpointer data;
   gsize size;
+  gboolean owned;
 
   g_return_val_if_fail (bitwriter != NULL, NULL);
 
-  size = bitwriter->bit_size >> 3;
+  owned = bitwriter->owned;
+
+  size = GST_ROUND_UP_8 (bitwriter->bit_size) >> 3;
   data = gst_bit_writer_reset_and_get_data (bitwriter);
 
   /* we cannot rely on buffers allocated externally, thus let's dup
    * the data */
-  if (data && !bitwriter->owned)
-    data = g_memdup (data, size);
+  if (data && !owned)
+    data = g_memdup2 (data, size);
 
   buffer = gst_buffer_new ();
   if (data != NULL) {
@@ -255,7 +260,7 @@ gst_bit_writer_free (GstBitWriter * bitwriter)
   g_return_if_fail (bitwriter != NULL);
 
   gst_bit_writer_reset (bitwriter);
-  g_slice_free (GstBitWriter, bitwriter);
+  g_free (bitwriter);
 }
 
 /**
@@ -278,7 +283,7 @@ gst_bit_writer_free_and_get_data (GstBitWriter * bitwriter)
   g_return_val_if_fail (bitwriter != NULL, NULL);
 
   data = gst_bit_writer_reset_and_get_data (bitwriter);
-  g_slice_free (GstBitWriter, bitwriter);
+  g_free (bitwriter);
 
   return data;
 }
@@ -303,7 +308,7 @@ gst_bit_writer_free_and_get_buffer (GstBitWriter * bitwriter)
   g_return_val_if_fail (bitwriter != NULL, NULL);
 
   buffer = gst_bit_writer_reset_and_get_buffer (bitwriter);
-  g_slice_free (GstBitWriter, bitwriter);
+  g_free (bitwriter);
 
   return buffer;
 }
@@ -328,7 +333,7 @@ gst_bit_writer_get_size (const GstBitWriter * bitwriter)
  *
  * Get written data pointer
  *
- * Returns: data pointer
+ * Returns: (array) (transfer none): data pointer
  */
 guint8 *
 gst_bit_writer_get_data (const GstBitWriter * bitwriter)
@@ -413,7 +418,7 @@ GST_BIT_WRITER_WRITE_BITS (64)
 /**
  * gst_bit_writer_put_bytes:
  * @bitwriter: a #GstBitWriter instance
- * @data: pointer of data to write
+ * @data: (array): pointer of data to write
  * @nbytes: number of bytes to write
  *
  * Write @nbytes bytes of @data to #GstBitWriter.
