@@ -22,6 +22,7 @@ pub mod unix {
         target_os = "macos",
         target_os = "solaris",
         target_os = "illumos",
+        target_os = "hurd",
     )))]
     compile_error!("Unsupported Operating System");
 
@@ -48,6 +49,7 @@ pub mod unix {
         target_os = "netbsd",
         target_os = "dragonfly",
         target_os = "macos",
+        target_os = "hurd",
     ))]
     pub const IP_ADD_MEMBERSHIP: c_int = 12;
     #[cfg(target_os = "linux")]
@@ -61,6 +63,7 @@ pub mod unix {
         target_os = "netbsd",
         target_os = "dragonfly",
         target_os = "macos",
+        target_os = "hurd",
     ))]
     pub const IP_MULTICAST_IF: c_int = 9;
     #[cfg(target_os = "linux")]
@@ -76,6 +79,7 @@ pub mod unix {
         target_os = "netbsd",
         target_os = "dragonfly",
         target_os = "macos",
+        target_os = "hurd",
     ))]
     pub const SOL_SOCKET: c_int = 0xffff;
     #[cfg(all(
@@ -119,6 +123,7 @@ pub mod unix {
         target_os = "netbsd",
         target_os = "dragonfly",
         target_os = "macos",
+        target_os = "hurd",
     ))]
     pub const SO_REUSEADDR: c_int = 0x4;
     #[cfg(all(
@@ -148,6 +153,7 @@ pub mod unix {
         target_os = "netbsd",
         target_os = "dragonfly",
         target_os = "macos",
+        target_os = "hurd",
     ))]
     pub const SO_REUSEPORT: c_int = 0x200;
     #[cfg(all(
@@ -171,7 +177,12 @@ pub mod unix {
     ))]
     pub const SO_REUSEPORT: c_int = 15;
 
-    #[cfg(any(target_os = "freebsd", target_os = "dragonfly", target_os = "netbsd"))]
+    #[cfg(any(
+        target_os = "freebsd",
+        target_os = "dragonfly",
+        target_os = "netbsd",
+        target_os = "hurd",
+    ))]
     pub const SOCK_CLOEXEC: c_int = 0x10000000;
     #[cfg(target_os = "openbsd")]
     pub const SOCK_CLOEXEC: c_int = 0x8000;
@@ -194,6 +205,7 @@ pub mod unix {
         target_os = "netbsd",
         target_os = "openbsd",
         target_os = "macos",
+        target_os = "hurd",
     ))]
     pub const SOCK_DGRAM: c_int = 2;
     #[cfg(any(target_os = "solaris", target_os = "illumos"))]
@@ -213,6 +225,7 @@ pub mod unix {
         target_os = "netbsd",
         target_os = "dragonfly",
         target_os = "macos",
+        target_os = "hurd",
     ))]
     pub const AF_LINK: c_int = 18;
     #[cfg(any(target_os = "solaris", target_os = "illumos"))]
@@ -220,13 +233,26 @@ pub mod unix {
     #[cfg(target_os = "linux")]
     pub const AF_PACKET: c_int = 17;
 
-    pub const IFF_UP: c_int = 0x1;
-    pub const IFF_LOOPBACK: c_int = 0x8;
-
-    #[cfg(target_os = "linux")]
-    pub const IFF_MULTICAST: c_int = 0x1000;
+    #[cfg(any(
+        target_os = "linux",
+        target_os = "freebsd",
+        target_os = "openbsd",
+        target_os = "netbsd",
+        target_os = "dragonfly",
+        target_os = "macos",
+        target_os = "hurd",
+    ))]
+    pub type IfaFlags = c_uint;
     #[cfg(any(target_os = "solaris", target_os = "illumos"))]
-    pub const IFF_MULTICAST: ::c_int = 0x0800;
+    pub type IfaFlags = u64;
+
+    pub const IFF_UP: IfaFlags = 0x1;
+    pub const IFF_LOOPBACK: IfaFlags = 0x8;
+
+    #[cfg(any(target_os = "linux", target_os = "hurd"))]
+    pub const IFF_MULTICAST: IfaFlags = 0x1000;
+    #[cfg(any(target_os = "solaris", target_os = "illumos"))]
+    pub const IFF_MULTICAST: IfaFlags = 0x0800;
     #[cfg(any(
         target_os = "freebsd",
         target_os = "openbsd",
@@ -234,11 +260,16 @@ pub mod unix {
         target_os = "dragonfly",
         target_os = "macos",
     ))]
-    pub const IFF_MULTICAST: c_int = 0x08000;
+    pub const IFF_MULTICAST: IfaFlags = 0x08000;
 
     #[cfg(any(target_os = "solaris", target_os = "illumos"))]
     pub const IF_NAMESIZE: usize = 32;
-    #[cfg(not(any(target_os = "linux", target_os = "solaris", target_os = "illumos")))]
+    #[cfg(not(any(
+        target_os = "linux",
+        target_os = "solaris",
+        target_os = "illumos",
+        target_os = "hurd",
+    )))]
     pub const IF_NAMESIZE: usize = 16;
 
     pub const PRIO_PROCESS: c_int = 0;
@@ -272,9 +303,7 @@ pub mod unix {
         )]
         pub fn poll(fds: *mut pollfd, nfds: nfds_t, timeout: c_int) -> c_int;
 
-        #[cfg(not(any(target_os = "solaris", target_os = "illumos")))]
         pub fn if_nametoindex(name: *const c_char) -> c_uint;
-        #[cfg(not(any(target_os = "solaris", target_os = "illumos")))]
         pub fn setsockopt(
             socket: c_int,
             level: c_int,
@@ -306,9 +335,19 @@ pub mod unix {
         pub fn pipe(pipefd: *mut i32) -> i32;
     }
 
-    #[cfg(any(target_os = "linux", target_os = "solaris", target_os = "illumos"))]
+    #[cfg(any(
+        target_os = "linux",
+        target_os = "solaris",
+        target_os = "illumos",
+        target_os = "hurd",
+    ))]
     pub type nfds_t = c_ulong;
-    #[cfg(not(any(target_os = "linux", target_os = "solaris", target_os = "illumos")))]
+    #[cfg(not(any(
+        target_os = "linux",
+        target_os = "solaris",
+        target_os = "illumos",
+        target_os = "hurd",
+    )))]
     pub type nfds_t = c_uint;
 
     #[repr(C)]
@@ -337,24 +376,33 @@ pub mod unix {
         pub imr_ifindex: c_int,
     }
 
+    // macOS uses ip_mreq instead of ip_mreqn in its latest version of the MAC Kernel for IP_ADD_MEMBERSHIP
+    #[cfg(target_os = "macos")]
+    #[repr(C)]
+    #[derive(Copy, Clone)]
+    pub struct ip_mreq {
+        pub imr_multiaddr: in_addr,
+        pub imr_address: in_addr,
+    }
+
     #[cfg(any(target_os = "solaris", target_os = "illumos"))]
     #[repr(C)]
     pub struct ifaddrs {
         pub ifa_next: *mut ifaddrs,
         pub ifa_name: *mut c_char,
-        pub ifa_flags: c_ulong,
+        pub ifa_flags: IfaFlags,
         pub ifa_addr: *mut sockaddr,
         pub ifa_netmask: *mut sockaddr,
         pub ifa_dstaddr: *mut sockaddr,
         pub ifa_data: *mut c_void,
     }
 
-    #[cfg(target_os = "linux")]
+    #[cfg(any(target_os = "linux", target_os = "hurd"))]
     #[repr(C)]
     pub struct ifaddrs {
         pub ifa_next: *mut ifaddrs,
         pub ifa_name: *mut c_char,
-        pub ifa_flags: c_uint,
+        pub ifa_flags: IfaFlags,
         pub ifa_addr: *mut sockaddr,
         pub ifa_netmask: *mut sockaddr,
         pub ifa_ifu: *mut sockaddr,
@@ -372,7 +420,7 @@ pub mod unix {
     pub struct ifaddrs {
         pub ifa_next: *mut ifaddrs,
         pub ifa_name: *mut c_char,
-        pub ifa_flags: c_uint,
+        pub ifa_flags: IfaFlags,
         pub ifa_addr: *mut sockaddr,
         pub ifa_netmask: *mut sockaddr,
         pub ifa_dstaddr: *mut sockaddr,
@@ -394,6 +442,7 @@ pub mod unix {
         target_os = "netbsd",
         target_os = "dragonfly",
         target_os = "macos",
+        target_os = "hurd",
     ))]
     #[repr(C)]
     pub struct sockaddr {
@@ -416,6 +465,7 @@ pub mod unix {
         target_os = "netbsd",
         target_os = "dragonfly",
         target_os = "macos",
+        target_os = "hurd",
     ))]
     #[repr(C)]
     pub struct sockaddr_in {
@@ -587,13 +637,14 @@ pub mod unix {
             target_os = "netbsd",
             target_os = "solaris",
             target_os = "illumos",
+            target_os = "hurd",
         ))]
         pub type clock_id_t = c_int;
 
         #[cfg(target_os = "dragonfly")]
         pub type clock_id_t = c_ulong;
 
-        #[cfg(any(target_os = "solaris", target_os = "illumos"))]
+        #[cfg(any(target_os = "solaris", target_os = "illumos", target_os = "hurd"))]
         pub type time_t = c_long;
 
         #[cfg(any(target_os = "openbsd", target_os = "netbsd", target_os = "dragonfly"))]
@@ -667,7 +718,7 @@ pub mod unix {
         #[cfg(any(target_os = "openbsd", target_os = "netbsd",))]
         pub const CLOCK_MONOTONIC: clock_id_t = 3;
 
-        #[cfg(target_os = "linux")]
+        #[cfg(any(target_os = "linux", target_os = "hurd"))]
         pub const CLOCK_MONOTONIC: clock_id_t = 1;
 
         extern "C" {
