@@ -44,6 +44,23 @@ G_BEGIN_DECLS
 typedef struct _GstValve GstValve;
 typedef struct _GstValveClass GstValveClass;
 
+/**
+ * GstValveDropMode:
+ * @GST_VALVE_DROP_MODE_DROP_ALL: Drop all buffers, buffer lists and events.
+ * @GST_VALVE_DROP_MODE_FORWARD_STICKY_EVENTS: Drop all buffers, buffer lists
+ *     and non-sticky events but forward sticky events.
+ * @GST_VALVE_DROP_MODE_TRANSFORM_TO_GAP: Drop all buffers, buffer lists and
+ *     non-sticky events but forward sticky events and forward GAP events for each
+ *     dropped buffer.
+ *
+ * Since: 1.20
+ */
+typedef enum {
+  GST_VALVE_DROP_MODE_DROP_ALL,
+  GST_VALVE_DROP_MODE_FORWARD_STICKY_EVENTS,
+  GST_VALVE_DROP_MODE_TRANSFORM_TO_GAP,
+} GstValveDropMode;
+
 /**
  * GstValve:
  *
@@ -55,7 +72,9 @@ struct _GstValve
   GstElement parent;
 
   /* atomic boolean */
-  volatile gint drop;
+  gint drop;
+
+  GstValveDropMode drop_mode;
 
   /* Protected by the stream lock */
   gboolean discont;
